@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  bg_block_info.c - bluegene block information from the db2 database.
  *
- *  $Id: bg_block_info.c 20097 2010-04-20 16:42:16Z da $
+ *  $Id: bg_block_info.c 22095 2011-01-15 00:14:40Z da $
  *****************************************************************************
  *  Copyright (C) 2004-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -87,21 +87,21 @@ static int _block_is_deallocating(bg_record_t *bg_record)
 	int jobid = bg_record->job_running;
 	char *user_name = NULL;
 
-	if(bg_record->modifying)
+	if (bg_record->modifying)
 		return SLURM_SUCCESS;
 
 	user_name = xstrdup(bg_conf->slurm_user_name);
-	if(remove_all_users(bg_record->bg_block_id, NULL)
-	   == REMOVE_USER_ERR) {
+	if (remove_all_users(bg_record->bg_block_id, NULL)
+	    == REMOVE_USER_ERR) {
 		error("Something happened removing "
 		      "users from block %s",
 		      bg_record->bg_block_id);
 	}
 
-	if(bg_record->target_name && bg_record->user_name) {
-		if(!strcmp(bg_record->target_name, user_name)) {
-			if(strcmp(bg_record->target_name, bg_record->user_name)
-			   || (jobid > NO_JOB_RUNNING)) {
+	if (bg_record->target_name && bg_record->user_name) {
+		if (!strcmp(bg_record->target_name, user_name)) {
+			if (strcmp(bg_record->target_name, bg_record->user_name)
+			    || (jobid > NO_JOB_RUNNING)) {
 				kill_job_struct_t *freeit =
 					xmalloc(sizeof(freeit));
 				freeit->jobid = jobid;
@@ -123,7 +123,7 @@ static int _block_is_deallocating(bg_record_t *bg_record)
 			      "for block %s.",
 			      bg_record->bg_block_id);
 		}
-	} else if(bg_record->user_name) {
+	} else if (bg_record->user_name) {
 		error("Target Name was not set "
 		      "not set for block %s.",
 		      bg_record->bg_block_id);
@@ -136,8 +136,8 @@ static int _block_is_deallocating(bg_record_t *bg_record)
 		bg_record->target_name = xstrdup(bg_record->user_name);
 	}
 
-	if(remove_from_bg_list(bg_lists->job_running, bg_record)
-	   == SLURM_SUCCESS)
+	if (remove_from_bg_list(bg_lists->job_running, bg_record)
+	    == SLURM_SUCCESS)
 		num_unused_cpus += bg_record->cpu_cnt;
 	remove_from_bg_list(bg_lists->booted, bg_record);
 
@@ -149,7 +149,7 @@ static void _destroy_kill_struct(void *object)
 {
 	kill_job_struct_t *freeit = (kill_job_struct_t *)object;
 
-	if(freeit) {
+	if (freeit) {
 		xfree(freeit);
 	}
 }
@@ -172,14 +172,14 @@ extern int block_ready(struct job_record *job_ptr)
 	char *block_id = NULL;
 	bg_record_t *bg_record = NULL;
 
-	rc = select_g_select_jobinfo_get(job_ptr->select_jobinfo,
-					 SELECT_JOBDATA_BLOCK_ID, &block_id);
+	rc = get_select_jobinfo(job_ptr->select_jobinfo->data,
+				SELECT_JOBDATA_BLOCK_ID, &block_id);
 	if (rc == SLURM_SUCCESS) {
 		slurm_mutex_lock(&block_state_mutex);
 		bg_record = find_bg_record_in_list(bg_lists->main, block_id);
 
-		if(bg_record) {
-			if(bg_record->job_running != job_ptr->job_id) {
+		if (bg_record) {
+			if (bg_record->job_running != job_ptr->job_id) {
 				rc = 0;
 			} else if ((bg_record->user_uid == job_ptr->user_id)
 				   && (bg_record->state
@@ -189,7 +189,7 @@ extern int block_ready(struct job_record *job_ptr)
 				*/
 				job_ptr->job_state &= (~JOB_CONFIGURING);
 				last_job_update = time(NULL);
-				
+
 				rc = 1;
 			} else if (bg_record->user_uid != job_ptr->user_id)
 				rc = 0;
@@ -201,41 +201,65 @@ extern int block_ready(struct job_record *job_ptr)
 			   often during an epilog on a busy system.
 			*/
 			debug2("block_ready: block %s not in bg_lists->main.",
-			      block_id);
+			       block_id);
 			rc = READY_JOB_FATAL;	/* fatal error */
 		}
 		slurm_mutex_unlock(&block_state_mutex);
-		xfree(block_id);
 	} else
 		rc = READY_JOB_ERROR;
-/* 	info("returning %d for job %u %d %d", */
-/* 	     rc, job_ptr->job_id, READY_JOB_ERROR, READY_JOB_FATAL); */
+	/* info("returning %d for job %u block %s %d %d", */
+	/*      rc, job_ptr->job_id, block_id, */
+	/*      READY_JOB_ERROR, READY_JOB_FATAL); */
+	xfree(block_id);
 	return rc;
 }
 
 /* Pack all relevent information about a block */
-extern void pack_block(bg_record_t *bg_record, Buf buffer)
+extern void pack_block(bg_record_t *bg_record, Buf buffer,
+		       uint16_t protocol_version)
 {
-	packstr(bg_record->bg_block_id, buffer);
+	if (protocol_version >= SLURM_2_2_PROTOCOL_VERSION) {
+		packstr(bg_record->bg_block_id, buffer);
 #ifdef HAVE_BGL
-	packstr(bg_record->blrtsimage, buffer);
+		packstr(bg_record->blrtsimage, buffer);
 #endif
-	pack_bit_fmt(bg_record->bitmap, buffer);
-	pack16((uint16_t)bg_record->conn_type, buffer);
-	packstr(bg_record->ionodes, buffer);
-	pack_bit_fmt(bg_record->ionode_bitmap, buffer);
-	pack32((uint32_t)bg_record->job_running, buffer);
-	packstr(bg_record->linuximage, buffer);
-	packstr(bg_record->mloaderimage, buffer);
-	packstr(bg_record->nodes, buffer);
-	pack32((uint32_t)bg_record->node_cnt, buffer);
+		pack_bit_fmt(bg_record->bitmap, buffer);
+		pack16((uint16_t)bg_record->conn_type, buffer);
+		packstr(bg_record->ionodes, buffer);
+		pack_bit_fmt(bg_record->ionode_bitmap, buffer);
+		pack32((uint32_t)bg_record->job_running, buffer);
+		packstr(bg_record->linuximage, buffer);
+		packstr(bg_record->mloaderimage, buffer);
+		packstr(bg_record->nodes, buffer);
+		pack32((uint32_t)bg_record->node_cnt, buffer);
 #ifdef HAVE_BGL
-	pack16((uint16_t)bg_record->node_use, buffer);
+		pack16((uint16_t)bg_record->node_use, buffer);
 #endif
-
-	packstr(bg_record->user_name, buffer);
-	packstr(bg_record->ramdiskimage, buffer);
-	pack16((uint16_t)bg_record->state, buffer);
+		packstr(bg_record->user_name, buffer);
+		packstr(bg_record->ramdiskimage, buffer);
+		packstr(bg_record->reason, buffer);
+		pack16((uint16_t)bg_record->state, buffer);
+	} else if (protocol_version >= SLURM_2_1_PROTOCOL_VERSION) {
+		packstr(bg_record->bg_block_id, buffer);
+#ifdef HAVE_BGL
+		packstr(bg_record->blrtsimage, buffer);
+#endif
+		pack_bit_fmt(bg_record->bitmap, buffer);
+		pack16((uint16_t)bg_record->conn_type, buffer);
+		packstr(bg_record->ionodes, buffer);
+		pack_bit_fmt(bg_record->ionode_bitmap, buffer);
+		pack32((uint32_t)bg_record->job_running, buffer);
+		packstr(bg_record->linuximage, buffer);
+		packstr(bg_record->mloaderimage, buffer);
+		packstr(bg_record->nodes, buffer);
+		pack32((uint32_t)bg_record->node_cnt, buffer);
+#ifdef HAVE_BGL
+		pack16((uint16_t)bg_record->node_use, buffer);
+#endif
+		packstr(bg_record->user_name, buffer);
+		packstr(bg_record->ramdiskimage, buffer);
+		pack16((uint16_t)bg_record->state, buffer);
+	}
 }
 
 extern int update_block_list()
@@ -250,27 +274,29 @@ extern int update_block_list()
 	rm_partition_state_t state;
 	char *name = NULL;
 	bg_record_t *bg_record = NULL;
-	time_t now;
 	kill_job_struct_t *freeit = NULL;
 	ListIterator itr = NULL;
-	slurmctld_lock_t job_write_lock = {
-		NO_LOCK, WRITE_LOCK, WRITE_LOCK, NO_LOCK };
 
-	if(!kill_job_list)
+	if (!kill_job_list)
 		kill_job_list = list_create(_destroy_kill_struct);
 
-	if(!bg_lists->main)
+	if (!bg_lists->main)
 		return updated;
 
 	slurm_mutex_lock(&block_state_mutex);
 	itr = list_iterator_create(bg_lists->main);
 	while ((bg_record = (bg_record_t *) list_next(itr)) != NULL) {
-		if(!bg_record->bg_block_id)
+		if (bg_record->magic != BLOCK_MAGIC) {
+			/* block is gone */
+			list_remove(itr);
 			continue;
+		} else if (!bg_record->bg_block_id)
+			continue;
+
 		name = bg_record->bg_block_id;
 		if ((rc = bridge_get_block_info(name, &block_ptr))
 		    != STATUS_OK) {
-			if(bg_conf->layout_mode == LAYOUT_DYNAMIC) {
+			if (bg_conf->layout_mode == LAYOUT_DYNAMIC) {
 				switch(rc) {
 				case INCONSISTENT_DATA:
 					debug2("got inconsistent data when "
@@ -293,7 +319,7 @@ extern int update_block_list()
 			   iteration.  It usually means something like
 			   rm_get_BG was called which can be a very
 			   long call */
-			if(rc == EBUSY) {
+			if (rc == EBUSY) {
 				debug5("lock was busy, aborting");
 				break;
 			}
@@ -312,7 +338,7 @@ extern int update_block_list()
 			      bg_err_str(rc));
 			updated = -1;
 			goto next_block;
-		} else if(bg_record->node_use != node_use) {
+		} else if (bg_record->node_use != node_use) {
 			debug("node_use of Block %s was %d "
 			      "and now is %d",
 			      bg_record->bg_block_id,
@@ -322,8 +348,8 @@ extern int update_block_list()
 			updated = 1;
 		}
 #else
-		if((bg_record->node_cnt < bg_conf->bp_node_cnt)
-		   || (bg_conf->bp_node_cnt == bg_conf->nodecard_node_cnt)) {
+		if ((bg_record->node_cnt < bg_conf->bp_node_cnt)
+		    || (bg_conf->bp_node_cnt == bg_conf->nodecard_node_cnt)) {
 			char *mode = NULL;
 			uint16_t conn_type = SELECT_SMALL;
 			if ((rc = bridge_get_data(block_ptr,
@@ -334,7 +360,7 @@ extern int update_block_list()
 				      "%s", bg_err_str(rc));
 				updated = -1;
 				goto next_block;
-			} else if(mode) {
+			} else if (mode) {
 				switch(mode[0]) {
 				case 's':
 					conn_type = SELECT_HTC_S;
@@ -355,7 +381,7 @@ extern int update_block_list()
 				free(mode);
 			}
 
-			if(bg_record->conn_type != conn_type) {
+			if (bg_record->conn_type != conn_type) {
 				debug("mode of small Block %s was %u "
 				      "and now is %u",
 				      bg_record->bg_block_id,
@@ -373,9 +399,9 @@ extern int update_block_list()
 			      bg_err_str(rc));
 			updated = -1;
 			goto next_block;
-		} else if(bg_record->job_running != BLOCK_ERROR_STATE
-			  //plugin set error
-			  && bg_record->state != state) {
+		} else if (bg_record->job_running != BLOCK_ERROR_STATE
+			   //plugin set error
+			   && bg_record->state != state) {
 			int skipped_dealloc = 0;
 
 			debug("state of Block %s was %d and now is %d",
@@ -383,15 +409,15 @@ extern int update_block_list()
 			      bg_record->state,
 			      state);
 			/*
-			   check to make sure block went
-			   through freeing correctly
+			  check to make sure block went
+			  through freeing correctly
 			*/
-			if((bg_record->state != RM_PARTITION_DEALLOCATING
-			    && bg_record->state != RM_PARTITION_ERROR)
-			   && state == RM_PARTITION_FREE)
+			if ((bg_record->state != RM_PARTITION_DEALLOCATING
+			     && bg_record->state != RM_PARTITION_ERROR)
+			    && state == RM_PARTITION_FREE)
 				skipped_dealloc = 1;
-			else if((bg_record->state == RM_PARTITION_READY)
-				&& (state == RM_PARTITION_CONFIGURING)) {
+			else if ((bg_record->state == RM_PARTITION_READY)
+				 && (state == RM_PARTITION_CONFIGURING)) {
 				/* This means the user did a reboot through
 				   mpirun but we missed the state
 				   change */
@@ -403,9 +429,9 @@ extern int update_block_list()
 				xfree(bg_record->target_name);
 				bg_record->target_name =
 					xstrdup(bg_record->user_name);
-			} else if((bg_record->state
-				   == RM_PARTITION_DEALLOCATING)
-				  && (state == RM_PARTITION_CONFIGURING))
+			} else if ((bg_record->state
+				    == RM_PARTITION_DEALLOCATING)
+				   && (state == RM_PARTITION_CONFIGURING))
 				/* This is a funky state IBM says
 				   isn't a bug, but all their
 				   documentation says this doesn't
@@ -417,11 +443,11 @@ extern int update_block_list()
 
 			bg_record->state = state;
 
-			if(bg_record->state == RM_PARTITION_DEALLOCATING
-			   || skipped_dealloc)
+			if (bg_record->state == RM_PARTITION_DEALLOCATING
+			    || skipped_dealloc)
 				_block_is_deallocating(bg_record);
 #ifndef HAVE_BGL
-			else if(bg_record->state == RM_PARTITION_REBOOTING) {
+			else if (bg_record->state == RM_PARTITION_REBOOTING) {
 				/* This means the user did a reboot through
 				   mpirun */
 				debug("Block %s rebooting.  "
@@ -433,18 +459,19 @@ extern int update_block_list()
 					xstrdup(bg_record->user_name);
 			}
 #endif
-			else if(bg_record->state == RM_PARTITION_CONFIGURING)
+			else if (bg_record->state == RM_PARTITION_CONFIGURING) {
+				debug("Setting bootflag for %s",
+				      bg_record->bg_block_id);
 				bg_record->boot_state = 1;
-			else if(bg_record->state == RM_PARTITION_FREE) {
-				if(remove_from_bg_list(bg_lists->job_running,
-						       bg_record)
-				   == SLURM_SUCCESS) {
+			} else if (bg_record->state == RM_PARTITION_FREE) {
+				if (remove_from_bg_list(bg_lists->job_running,
+							bg_record)
+				    == SLURM_SUCCESS)
 					num_unused_cpus += bg_record->cpu_cnt;
-				}
 				remove_from_bg_list(bg_lists->booted,
 						    bg_record);
-			} else if(bg_record->state == RM_PARTITION_ERROR) {
-				if(bg_record->boot_state == 1)
+			} else if (bg_record->state == RM_PARTITION_ERROR) {
+				if (bg_record->boot_state == 1)
 					error("Block %s in an error "
 					      "state while booting.",
 					      bg_record->bg_block_id);
@@ -454,7 +481,12 @@ extern int update_block_list()
 				remove_from_bg_list(bg_lists->booted,
 						    bg_record);
 				trigger_block_error();
+			} else if (bg_record->state == RM_PARTITION_READY) {
+				if (!block_ptr_exist_in_list(bg_lists->booted,
+							     bg_record))
+					list_push(bg_lists->booted, bg_record);
 			}
+
 			updated = 1;
 		}
 	nochange_state:
@@ -463,16 +495,16 @@ extern int update_block_list()
 		debug3("boot state for block %s is %d",
 		       bg_record->bg_block_id,
 		       bg_record->boot_state);
-		if(bg_record->boot_state == 1) {
+		if (bg_record->boot_state == 1) {
 			switch(bg_record->state) {
 			case RM_PARTITION_CONFIGURING:
 				debug3("checking to make sure user %s "
 				       "is the user.",
 				       bg_record->target_name);
 
-				if(update_block_user(bg_record, 0) == 1)
+				if (update_block_user(bg_record, 0) == 1)
 					last_bg_update = time(NULL);
-				if(bg_record->job_ptr) {
+				if (bg_record->job_ptr) {
 					bg_record->job_ptr->job_state |=
 						JOB_CONFIGURING;
 					last_job_update = time(NULL);
@@ -489,62 +521,54 @@ extern int update_block_list()
 				 */
 				break;
 			case RM_PARTITION_FREE:
-				if(bg_record->boot_count < RETRY_BOOT_COUNT) {
-					slurm_mutex_unlock(&block_state_mutex);
-					if((rc = boot_block(bg_record))
-					   != SLURM_SUCCESS) {
+				if (bg_record->boot_count < RETRY_BOOT_COUNT) {
+					if ((rc = boot_block(bg_record))
+					    != SLURM_SUCCESS)
 						updated = -1;
+
+					if (bg_record->magic == BLOCK_MAGIC) {
+						debug("boot count for block "
+						      "%s is %d",
+						      bg_record->bg_block_id,
+						      bg_record->boot_count);
+						bg_record->boot_count++;
 					}
-					slurm_mutex_lock(&block_state_mutex);
-					debug("boot count for block "
-					      "%s is %d",
-					      bg_record->bg_block_id,
-					      bg_record->boot_count);
-					bg_record->boot_count++;
 				} else {
-					char reason[128], time_str[32];
+					char *reason = "update_block_list: "
+						"Boot fails ";
 
 					error("Couldn't boot Block %s "
 					      "for user %s",
 					      bg_record->bg_block_id,
 					      bg_record->target_name);
-					slurm_mutex_unlock(&block_state_mutex);
 
-					now = time(NULL);
-					slurm_make_time_str(&now, time_str,
-							    sizeof(time_str));
-					snprintf(reason,
-						 sizeof(reason),
-						 "update_block_list: "
-						 "Boot fails "
-						 "[SLURM@%s]",
-						 time_str);
-					drain_as_needed(bg_record, reason);
+					slurm_mutex_unlock(&block_state_mutex);
+					requeue_and_error(bg_record, reason);
 					slurm_mutex_lock(&block_state_mutex);
+
 					bg_record->boot_state = 0;
 					bg_record->boot_count = 0;
-					if(remove_from_bg_list(
-						   bg_lists->job_running,
-						   bg_record)
-					   == SLURM_SUCCESS) {
+					if (remove_from_bg_list(
+						    bg_lists->job_running,
+						    bg_record)
+					    == SLURM_SUCCESS) {
 						num_unused_cpus +=
 							bg_record->cpu_cnt;
 					}
 					remove_from_bg_list(
-						bg_lists->booted,
-						bg_record);
+						bg_lists->booted, bg_record);
 				}
 				break;
 			case RM_PARTITION_READY:
 				debug("block %s is ready.",
 				      bg_record->bg_block_id);
-				if(bg_record->job_ptr) {
+				if (bg_record->job_ptr) {
 					bg_record->job_ptr->job_state &=
 						(~JOB_CONFIGURING);
 					last_job_update = time(NULL);
 				}
-
-				if(set_block_user(bg_record) == SLURM_ERROR) {
+				/* boot flags are reset here */
+				if (set_block_user(bg_record) == SLURM_ERROR) {
 					freeit = xmalloc(
 						sizeof(kill_job_struct_t));
 					freeit->jobid = bg_record->job_running;
@@ -583,16 +607,9 @@ extern int update_block_list()
 	slurm_mutex_unlock(&block_state_mutex);
 
 	/* kill all the jobs from unexpectedly freed blocks */
-	while((freeit = list_pop(kill_job_list))) {
-		debug2("Trying to requeue job %d", freeit->jobid);
-		lock_slurmctld(job_write_lock);
-		if((rc = job_requeue(0, freeit->jobid, -1))) {
-			error("couldn't requeue job %u, failing it: %s",
-			      freeit->jobid,
-			      slurm_strerror(rc));
-			(void) job_fail(freeit->jobid);
-		}
-		unlock_slurmctld(job_write_lock);
+	while ((freeit = list_pop(kill_job_list))) {
+		debug2("Trying to requeue job %u", freeit->jobid);
+		bg_requeue_job(freeit->jobid, 0);
 		_destroy_kill_struct(freeit);
 	}
 
@@ -600,7 +617,8 @@ extern int update_block_list()
 	return updated;
 }
 
-extern int update_freeing_block_list()
+/* This needs to have block_state_mutex locked before hand. */
+extern int update_block_list_state(List block_list)
 {
 	int updated = 0;
 #ifdef HAVE_BG_FILES
@@ -611,19 +629,19 @@ extern int update_freeing_block_list()
 	bg_record_t *bg_record = NULL;
 	ListIterator itr = NULL;
 
-	if(!bg_lists->freeing)
-		return updated;
-
-	slurm_mutex_lock(&block_state_mutex);
-	itr = list_iterator_create(bg_lists->freeing);
+	itr = list_iterator_create(block_list);
 	while ((bg_record = (bg_record_t *) list_next(itr)) != NULL) {
-		if(!bg_record->bg_block_id)
+		if (bg_record->magic != BLOCK_MAGIC) {
+			/* block is gone */
+			list_remove(itr);
+			continue;
+		} else if (!bg_record->bg_block_id)
 			continue;
 
 		name = bg_record->bg_block_id;
 		if ((rc = bridge_get_block_info(name, &block_ptr))
 		    != STATUS_OK) {
-			if(bg_conf->layout_mode == LAYOUT_DYNAMIC) {
+			if (bg_conf->layout_mode == LAYOUT_DYNAMIC) {
 				switch(rc) {
 				case INCONSISTENT_DATA:
 					debug2("got inconsistent data when "
@@ -633,8 +651,11 @@ extern int update_freeing_block_list()
 				case PARTITION_NOT_FOUND:
 					debug("block %s not found, removing "
 					      "from slurm", name);
-					list_remove(itr);
-					destroy_bg_record(bg_record);
+					/* Just set to free,
+					   everything will be cleaned
+					   up outside this.
+					*/
+					bg_record->state = RM_PARTITION_FREE;
 					continue;
 					break;
 				default:
@@ -645,7 +666,7 @@ extern int update_freeing_block_list()
 			   iteration.  It usually means something like
 			   rm_get_BG was called which can be a very
 			   long call */
-			if(rc == EBUSY) {
+			if (rc == EBUSY) {
 				debug5("lock was busy, aborting");
 				break;
 			}
@@ -663,7 +684,7 @@ extern int update_freeing_block_list()
 			      bg_err_str(rc));
 			updated = -1;
 			goto next_block;
-		} else if(bg_record->state != state) {
+		} else if (bg_record->state != state) {
 			debug("freeing state of Block %s was %d and now is %d",
 			      bg_record->bg_block_id,
 			      bg_record->state,
@@ -680,8 +701,6 @@ extern int update_freeing_block_list()
 		}
 	}
 	list_iterator_destroy(itr);
-	slurm_mutex_unlock(&block_state_mutex);
-
 #endif
 	return updated;
 }
