@@ -41,6 +41,18 @@
 #include "src/sacctmgr/sacctmgr.h"
 #include <sys/param.h>		/* MAXPATHLEN */
 #include "src/common/proc_args.h"
+#include "src/common/uid.h"
+
+static char *_string_to_uid( char *name )
+{
+	uid_t uid;
+	if ( uid_from_string( name, &uid ) != 0 ) {
+		fprintf(stderr, "Invalid user id: %s\n", name);
+		exit(1);
+	}
+	xfree(name);
+	return xstrdup_printf( "%d", (int) uid );
+}
 
 /* returns number of objects added to list */
 extern int _addto_uid_char_list(List char_list, char *names)
@@ -76,19 +88,7 @@ extern int _addto_uid_char_list(List char_list, char *names)
 					name = xmalloc((i-start+1));
 					memcpy(name, names+start, (i-start));
 					//info("got %s %d", name, i-start);
-					if (!isdigit((int) *name)) {
-						struct passwd *pwd;
-						if (!(pwd=getpwnam(name))) {
-							fprintf(stderr,
-								"Invalid user "
-								"id: %s\n",
-								name);
-							exit(1);
-						}
-						xfree(name);
-						name = xstrdup_printf(
-							"%d", pwd->pw_uid);
-					}
+					name = _string_to_uid( name );
 
 					while((tmp_char = list_next(itr))) {
 						if(!strcasecmp(tmp_char, name))
@@ -116,18 +116,7 @@ extern int _addto_uid_char_list(List char_list, char *names)
 		if((i-start) > 0) {
 			name = xmalloc((i-start)+1);
 			memcpy(name, names+start, (i-start));
-
-			if (!isdigit((int) *name)) {
-				struct passwd *pwd;
-				if (!(pwd=getpwnam(name))) {
-					fprintf(stderr,
-						"Invalid user id: %s\n",
-						name);
-					exit(1);
-				}
-				xfree(name);
-				name = xstrdup_printf("%d", pwd->pw_uid);
-			}
+			name = _string_to_uid( name );
 
 			while((tmp_char = list_next(itr))) {
 				if(!strcasecmp(tmp_char, name))
@@ -146,21 +135,22 @@ extern int _addto_uid_char_list(List char_list, char *names)
 }
 
 static int _set_cond(int *start, int argc, char *argv[],
-		     acct_archive_cond_t *arch_cond)
+		     slurmdb_archive_cond_t *arch_cond)
 {
 	int i;
 	int set = 0;
 	int end = 0;
 	int command_len = 0;
 	int option = 0;
-	acct_job_cond_t *job_cond = NULL;
+ 	uint32_t tmp;
+	slurmdb_job_cond_t *job_cond = NULL;
 
 	if(!arch_cond) {
 		error("No arch_cond given");
 		return -1;
 	}
 	if(!arch_cond->job_cond)
-		arch_cond->job_cond = xmalloc(sizeof(acct_job_cond_t));
+		arch_cond->job_cond = xmalloc(sizeof(slurmdb_job_cond_t));
 	job_cond = arch_cond->job_cond;
 
 	for (i=(*start); i<argc; i++) {
@@ -180,23 +170,26 @@ static int _set_cond(int *start, int argc, char *argv[],
 			continue;
 		} else if(!end && !strncasecmp(argv[i], "events",
 					  MAX(command_len, 1))) {
-			arch_cond->archive_events = 1;
+			arch_cond->purge_event |= SLURMDB_PURGE_ARCHIVE;
 			set = 1;
 		} else if(!end && !strncasecmp(argv[i], "jobs",
 					  MAX(command_len, 1))) {
-			arch_cond->archive_jobs = 1;
+			arch_cond->purge_job |= SLURMDB_PURGE_ARCHIVE;
 			set = 1;
 		} else if(!end && !strncasecmp(argv[i], "steps",
 					  MAX(command_len, 1))) {
-			arch_cond->archive_steps = 1;
+			arch_cond->purge_step |= SLURMDB_PURGE_ARCHIVE;
 			set = 1;
 		} else if(!end && !strncasecmp(argv[i], "suspend",
 					  MAX(command_len, 1))) {
-			arch_cond->archive_suspend = 1;
+			arch_cond->purge_suspend |= SLURMDB_PURGE_ARCHIVE;
 			set = 1;
 		} else if(!end
-			  || !strncasecmp (argv[i], "Clusters",
-					   MAX(command_len, 1))) {
+			  || !strncasecmp(argv[i], "Clusters",
+					  MAX(command_len, 1))) {
+			if(!job_cond->cluster_list)
+				job_cond->cluster_list =
+					list_create(slurm_destroy_char);
 			slurm_addto_char_list(job_cond->cluster_list,
 					      argv[i]+end);
 			set = 1;
@@ -234,7 +227,7 @@ static int _set_cond(int *start, int argc, char *argv[],
 		} else if (!strncasecmp (argv[i], "Jobs",
 					 MAX(command_len, 1))) {
 			char *end_char = NULL, *start_char = argv[i]+end;
-			jobacct_selected_step_t *selected_step = NULL;
+			slurmdb_selected_step_t *selected_step = NULL;
 			char *dot = NULL;
 			if(!job_cond->step_list)
 				job_cond->step_list =
@@ -248,7 +241,7 @@ static int _set_cond(int *start, int argc, char *argv[],
 				if(!(int)*start_char)
 					continue;
 				selected_step = xmalloc(
-					sizeof(jobacct_selected_step_t));
+					sizeof(slurmdb_selected_step_t));
 				list_append(job_cond->step_list, selected_step);
 
 				dot = strstr(start_char, ".");
@@ -272,38 +265,83 @@ static int _set_cond(int *start, int argc, char *argv[],
 			slurm_addto_char_list(job_cond->partition_list,
 					      argv[i]+end);
 			set = 1;
+		} else if (!strncasecmp (argv[i], "PurgeEventAfter",
+					 MAX(command_len, 10))) {
+			if ((tmp = slurmdb_parse_purge(argv[i]+end))
+			    == NO_VAL) {
+				exit_code = 1;
+			} else {
+				arch_cond->purge_event |= tmp;
+				set = 1;
+			}
+		} else if (!strncasecmp (argv[i], "PurgeJobAfter",
+					 MAX(command_len, 10))) {
+			if ((tmp = slurmdb_parse_purge(argv[i]+end))
+			    == NO_VAL) {
+				exit_code = 1;
+			} else {
+				arch_cond->purge_job |= tmp;
+				set = 1;
+			}
+		} else if (!strncasecmp (argv[i], "PurgeStepAfter",
+					 MAX(command_len, 10))) {
+			if ((tmp = slurmdb_parse_purge(argv[i]+end))
+			    == NO_VAL) {
+				exit_code = 1;
+			} else {
+				arch_cond->purge_step |= tmp;
+				set = 1;
+			}
+		} else if (!strncasecmp (argv[i], "PurgeSuspendAfter",
+					 MAX(command_len, 10))) {
+			if ((tmp = slurmdb_parse_purge(argv[i]+end))
+			    == NO_VAL) {
+				exit_code = 1;
+			} else {
+				arch_cond->purge_suspend |= tmp;
+				set = 1;
+			}
 		} else if (!strncasecmp (argv[i], "PurgeEventMonths",
 					 MAX(command_len, 6))) {
-			if (get_uint16(argv[i]+end, &arch_cond->purge_event,
-				       "PurgeEventMonths")
+			if (get_uint(argv[i]+end, &tmp, "PurgeEventMonths")
 			    != SLURM_SUCCESS) {
 				exit_code = 1;
-			} else
+			} else {
+				arch_cond->purge_event |= tmp;
+				arch_cond->purge_event |= SLURMDB_PURGE_MONTHS;
 				set = 1;
+			}
 		} else if (!strncasecmp (argv[i], "PurgeJobMonths",
 					 MAX(command_len, 6))) {
-			if (get_uint16(argv[i]+end, &arch_cond->purge_job,
-				       "PurgeJobMonths")
+			if (get_uint(argv[i]+end, &tmp, "PurgeJobMonths")
 			    != SLURM_SUCCESS) {
 				exit_code = 1;
-			} else
+			} else {
+				arch_cond->purge_job |= tmp;
+				arch_cond->purge_job |= SLURMDB_PURGE_MONTHS;
 				set = 1;
+			}
 		} else if (!strncasecmp (argv[i], "PurgeStepMonths",
 					 MAX(command_len, 7))) {
-			if (get_uint16(argv[i]+end, &arch_cond->purge_step,
-				       "PurgeStepMonths")
+			if (get_uint(argv[i]+end, &tmp, "PurgeStepMonths")
 			    != SLURM_SUCCESS) {
 				exit_code = 1;
-			} else
+			} else {
+				arch_cond->purge_step |= tmp;
+				arch_cond->purge_step |= SLURMDB_PURGE_MONTHS;
 				set = 1;
+			}
 		} else if (!strncasecmp (argv[i], "PurgeSuspendMonths",
 					 MAX(command_len, 7))) {
-			if (get_uint16(argv[i]+end, &arch_cond->purge_suspend,
-				       "PurgeSuspendMonths")
+			if (get_uint(argv[i]+end, &tmp, "PurgeSuspendMonths")
 			    != SLURM_SUCCESS) {
 				exit_code = 1;
-			} else
+			} else {
+				arch_cond->purge_suspend |= tmp;
+				arch_cond->purge_suspend
+					|= SLURMDB_PURGE_MONTHS;
 				set = 1;
+			}
 		} else if (!strncasecmp (argv[i], "Start",
 					 MAX(command_len, 2))) {
 			job_cond->usage_start = parse_time(argv[i]+end, 1);
@@ -335,19 +373,11 @@ static int _set_cond(int *start, int argc, char *argv[],
 extern int sacctmgr_archive_dump(int argc, char *argv[])
 {
 	int rc = SLURM_SUCCESS;
-	acct_archive_cond_t *arch_cond = xmalloc(sizeof(acct_archive_cond_t));
+	slurmdb_archive_cond_t *arch_cond =
+		xmalloc(sizeof(slurmdb_archive_cond_t));
 	int i=0;
 	struct stat st;
 
-	arch_cond->archive_events = (uint16_t)NO_VAL;
-	arch_cond->archive_jobs = (uint16_t)NO_VAL;
-	arch_cond->archive_steps = (uint16_t)NO_VAL;
-	arch_cond->archive_suspend = (uint16_t)NO_VAL;
-	arch_cond->purge_event = (uint16_t)NO_VAL;
-	arch_cond->purge_job = (uint16_t)NO_VAL;
-	arch_cond->purge_step = (uint16_t)NO_VAL;
-	arch_cond->purge_suspend = (uint16_t)NO_VAL;
-
 	for (i=0; i<argc; i++) {
 		int command_len = strlen(argv[i]);
 		if (!strncasecmp (argv[i], "Where", MAX(command_len, 5))
@@ -356,8 +386,17 @@ extern int sacctmgr_archive_dump(int argc, char *argv[])
 		_set_cond(&i, argc, argv, arch_cond);
 	}
 
+	if(!arch_cond->purge_event)
+		arch_cond->purge_event = NO_VAL;
+	if(!arch_cond->purge_job)
+		arch_cond->purge_job = NO_VAL;
+	if(!arch_cond->purge_step)
+		arch_cond->purge_step = NO_VAL;
+	if(!arch_cond->purge_suspend)
+		arch_cond->purge_suspend = NO_VAL;
+
 	if(exit_code) {
-		destroy_acct_archive_cond(arch_cond);
+		slurmdb_destroy_archive_cond(arch_cond);
 		return SLURM_ERROR;
 	}
 
@@ -429,7 +468,7 @@ extern int sacctmgr_archive_dump(int argc, char *argv[])
 			slurm_strerror(rc));
 		rc = SLURM_ERROR;
 	}
-	destroy_acct_archive_cond(arch_cond);
+	slurmdb_destroy_archive_cond(arch_cond);
 
 	return rc;
 }
@@ -437,7 +476,8 @@ extern int sacctmgr_archive_dump(int argc, char *argv[])
 extern int sacctmgr_archive_load(int argc, char *argv[])
 {
 	int rc = SLURM_SUCCESS;
-	acct_archive_rec_t *arch_rec = xmalloc(sizeof(acct_archive_rec_t));
+	slurmdb_archive_rec_t *arch_rec =
+		xmalloc(sizeof(slurmdb_archive_rec_t));
 	int i=0, command_len = 0, option = 0;
 	struct stat st;
 
@@ -467,7 +507,7 @@ extern int sacctmgr_archive_load(int argc, char *argv[])
 	}
 
 	if(exit_code) {
-		destroy_acct_archive_rec(arch_rec);
+		slurmdb_destroy_archive_rec(arch_rec);
 		return SLURM_ERROR;
 	}
 
@@ -510,7 +550,7 @@ extern int sacctmgr_archive_load(int argc, char *argv[])
 		rc = SLURM_ERROR;
 	}
 
-	destroy_acct_archive_rec(arch_rec);
+	slurmdb_destroy_archive_rec(arch_rec);
 
 	return rc;
 }
