@@ -46,6 +46,10 @@
 #  include "config.h"
 #endif
 
+#ifndef   _GNU_SOURCE
+#  define _GNU_SOURCE
+#endif
+
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
@@ -174,6 +178,7 @@ static void _do_power_work(void)
 		    (susp_state == 0)					&&
 		    ((suspend_rate == 0) || (suspend_cnt < suspend_rate)) &&
 		    IS_NODE_IDLE(node_ptr)				&&
+		    (node_ptr->sus_job_cnt == 0)			&&
 		    (!IS_NODE_COMPLETING(node_ptr))			&&
 		    (!IS_NODE_POWER_UP(node_ptr))			&&
 		    (node_ptr->last_idle < (now - idle_time))		&&
@@ -206,7 +211,7 @@ static void _do_power_work(void)
 		else
 			error("power_save: bitmap2nodename");
 		xfree(nodes);
-		bit_free(sleep_node_bitmap);
+		FREE_NULL_BITMAP(sleep_node_bitmap);
 		last_node_update = now;
 	}
 
@@ -218,7 +223,7 @@ static void _do_power_work(void)
 		else
 			error("power_save: bitmap2nodename");
 		xfree(nodes);
-		bit_free(wake_node_bitmap);
+		FREE_NULL_BITMAP(wake_node_bitmap);
 		last_node_update = now;
 	}
 }
@@ -256,7 +261,7 @@ static void _re_wake(void)
 		} else
 			error("power_save: bitmap2nodename");
 		xfree(nodes);
-		bit_free(wake_node_bitmap);
+		FREE_NULL_BITMAP(wake_node_bitmap);
 	}
 }
 
@@ -657,5 +662,6 @@ fini:	_clear_power_config();
 	slurm_mutex_lock(&power_mutex);
 	power_save_enabled = false;
 	slurm_mutex_unlock(&power_mutex);
+	pthread_exit(NULL);
 	return NULL;
 }
