@@ -2,7 +2,7 @@
  *  print.h - squeue print job definitions
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Copyright (C) 2008-2009 Lawrence Livermore National Security
+ *  Copyright (C) 2008-2010 Lawrence Livermore National Security
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Joey Ekstrom <ekstrom1@llnl.gov>
  *  CODE-OCEC-09-009. All rights reserved.
@@ -99,6 +99,8 @@ int job_format_add_function(List list, int width, bool right_justify,
 	job_format_add_function(list,wid,right,suffix,_print_job_user_name)
 #define job_format_add_user_id(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_user_id)
+#define job_format_add_gres(list,wid,right,suffix) \
+        job_format_add_function(list,wid,right,suffix,_print_job_gres)
 #define job_format_add_group_name(list,wid,right,suffix) \
         job_format_add_function(list,wid,right,suffix,_print_job_group_name)
 #define job_format_add_group_id(list,wid,right,suffix) \
@@ -128,8 +130,8 @@ int job_format_add_function(List list, int width, bool right_justify,
 	job_format_add_function(list,wid,right,suffix,_print_job_nodes)
 #define job_format_add_node_inx(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_node_inx)
-#define job_format_add_num_procs(list,wid,right,suffix) \
-	job_format_add_function(list,wid,right,suffix,_print_job_num_procs)
+#define job_format_add_num_cpus(list,wid,right,suffix) \
+	job_format_add_function(list,wid,right,suffix,_print_job_num_cpus)
 #define job_format_add_num_nodes(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_num_nodes)
 #define job_format_add_num_sct(list,wid,right,suffix) \
@@ -138,18 +140,18 @@ int job_format_add_function(List list, int width, bool right_justify,
 	job_format_add_function(list,wid,right,suffix,_print_job_shared)
 #define job_format_add_contiguous(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_contiguous)
-#define job_format_add_min_procs(list,wid,right,suffix) \
-	job_format_add_function(list,wid,right,suffix,_print_job_min_procs)
-#define job_format_add_min_sockets(list,wid,right,suffix) \
-	job_format_add_function(list,wid,right,suffix,_print_job_min_sockets)
-#define job_format_add_min_cores(list,wid,right,suffix) \
-	job_format_add_function(list,wid,right,suffix,_print_job_min_cores)
-#define job_format_add_min_threads(list,wid,right,suffix) \
-	job_format_add_function(list,wid,right,suffix,_print_job_min_threads)
+#define job_format_add_min_cpus(list,wid,right,suffix) \
+	job_format_add_function(list,wid,right,suffix,_print_pn_min_cpus)
+#define job_format_add_sockets(list,wid,right,suffix) \
+	job_format_add_function(list,wid,right,suffix,_print_sockets)
+#define job_format_add_cores(list,wid,right,suffix) \
+	job_format_add_function(list,wid,right,suffix,_print_cores)
+#define job_format_add_threads(list,wid,right,suffix) \
+	job_format_add_function(list,wid,right,suffix,_print_threads)
 #define job_format_add_min_memory(list,wid,right,suffix) \
-	job_format_add_function(list,wid,right,suffix,_print_job_min_memory)
+	job_format_add_function(list,wid,right,suffix,_print_pn_min_memory)
 #define job_format_add_min_tmp_disk(list,wid,right,suffix) \
-	job_format_add_function(list,wid,right,suffix,_print_job_min_tmp_disk)
+	job_format_add_function(list,wid,right,suffix,_print_pn_min_tmp_disk)
 #define job_format_add_req_nodes(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_req_nodes)
 #define job_format_add_exc_nodes(list,wid,right,suffix) \
@@ -196,6 +198,8 @@ int _print_job_user_id(job_info_t * job, int width, bool right_justify,
 			char* suffix);
 int _print_job_user_name(job_info_t * job, int width, bool right_justify,
 			char* suffix);
+int _print_job_gres(job_info_t * job, int width, bool right_justify,
+			char* suffix);
 int _print_job_group_id(job_info_t * job, int width, bool right_justify,
 			char* suffix);
 int _print_job_group_name(job_info_t * job, int width, bool right_justify,
@@ -224,7 +228,7 @@ int _print_job_node_inx(job_info_t * job, int width, bool right_justify,
 			char* suffix);
 int _print_job_partition(job_info_t * job, int width, bool right_justify,
 			char* suffix);
-int _print_job_num_procs(job_info_t * job, int width, bool right_justify,
+int _print_job_num_cpus(job_info_t * job, int width, bool right_justify,
 			char* suffix);
 int _print_job_num_nodes(job_info_t * job, int width, bool right_justify,
 			char* suffix);
@@ -234,17 +238,16 @@ int _print_job_shared(job_info_t * job, int width, bool right_justify,
 			char* suffix);
 int _print_job_contiguous(job_info_t * job, int width, bool right_justify,
 			char* suffix);
-int _print_job_min_procs(job_info_t * job, int width, bool right_justify,
+int _print_pn_min_cpus(job_info_t * job, int width, bool right_justify,
+		       char* suffix);
+int _print_sockets(job_info_t * job, int width, bool right_justify,
+		   char* suffix);
+int _print_cores(job_info_t * job, int width, bool right_justify, char* suffix);
+int _print_threads(job_info_t * job, int width, bool right_justify,
+		   char* suffix);
+int _print_pn_min_memory(job_info_t * job, int width, bool right_justify,
 			char* suffix);
-int _print_job_min_sockets(job_info_t * job, int width, bool right_justify,
-			char* suffix);
-int _print_job_min_cores(job_info_t * job, int width, bool right_justify,
-			char* suffix);
-int _print_job_min_threads(job_info_t * job, int width, bool right_justify,
-			char* suffix);
-int _print_job_min_memory(job_info_t * job, int width, bool right_justify,
-			char* suffix);
-int _print_job_min_tmp_disk(job_info_t * job, int width, bool right_justify,
+int _print_pn_min_tmp_disk(job_info_t * job, int width, bool right_justify,
 			char* suffix);
 int _print_job_req_nodes(job_info_t * job, int width, bool right_justify,
 			char* suffix);
@@ -298,6 +301,8 @@ int step_format_add_function(List list, int width, bool right_justify,
 	step_format_add_function(list,wid,right,suffix,_print_step_name)
 #define step_format_add_num_tasks(list,wid,right,suffix) \
 	step_format_add_function(list,wid,right,suffix,_print_step_num_tasks)
+#define step_format_add_gres(list,wid,right,suffix) \
+	step_format_add_function(list,wid,right,suffix,_print_step_gres)
 #define step_format_add_invalid(list,wid,right,suffix) \
 	step_format_add_function(list,wid,right,suffix,	\
 					(void*)_print_com_invalid)
@@ -327,6 +332,8 @@ int _print_step_nodes(job_step_info_t * step, int width,
 			bool right_justify, char *suffix);
 int _print_step_num_tasks(job_step_info_t * step, int width,
 			bool right_justify, char *suffix);
+int _print_step_gres(job_step_info_t * step, int width,
+			bool right_justify, char *suffix);
 
 /*****************************************************************************
  * Common Line Print Functions
